package buildcraft.energy.tile;

import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import com.mojang.authlib.GameProfile;

import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.nbt.NBTUtil;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;

import net.minecraftforge.common.util.Constants;

import buildcraft.api.core.BCLog;
import buildcraft.api.tiles.IDebuggable;

import buildcraft.core.tile.ITileOilSpring;

// We don't extend TileBC here because we have no need of any of its functions.
public class TileSpringOil extends TileEntity implements IDebuggable, ITileOilSpring {

    private final Map<GameProfile, PlayerPumpInfo> pumpProgress = new ConcurrentHashMap<>();

    /** An approximation of the total number of oil source blocks in the oil spring. The actual number will be less than
     * this, so this is taken as an approximation.
     * <p>
     * Note that this SHOULD NEVER be set! (Except by the generator, and readFromNbt) */
    public int totalSources;

    @Override
    public void onPumpOil(GameProfile profile, BlockPos oilPos) {
        if (profile == null) {
            // BCLog.logger.warn("Unknown owner for pump at " + pump.getPos());
            return;
        }
        PlayerPumpInfo info = pumpProgress.computeIfAbsent(profile, PlayerPumpInfo::new);
        info.lastPumpTick = field_145850_b.func_82737_E();
        info.sourcesPumped++;

        String name = profile.getName();
        // BCLog.logger.info("Pumped " + info.sourcesPumped + " / " + totalSources + " at " + oilPos + " (for " +
        // System.identityHashCode(this) + ", "+getPos()+")");
        if (info.sourcesPumped >= totalSources * 7 / 8) {
            // BCLog.logger.info("Pumped nearly all oil blocks!");
            if (oilPos.equals(func_174877_v().func_177984_a())) {
                BCLog.logger.info("Awarding advancement to " + name + "! Or we would do, if this was 1.12...");
            }
        }
    }

    @Override
    public void func_145839_a(NBTTagCompound nbt) {
        super.func_145839_a(nbt);
        NBTTagList list = nbt.func_150295_c("pumpProgress", Constants.NBT.TAG_COMPOUND);
        for (int i = 0; i < list.func_74745_c(); i++) {
            PlayerPumpInfo info = new PlayerPumpInfo(list.func_150305_b(i));
            pumpProgress.put(info.profile, info);
        }
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound nbt) {
        super.func_189515_b(nbt);
        nbt.func_74768_a("totalSources", totalSources);
        NBTTagList list = new NBTTagList();
        for (PlayerPumpInfo info : pumpProgress.values()) {
            list.func_74742_a(info.writeToNbt());
        }
        nbt.func_74782_a("pumpProgress", list);
        return nbt;
    }

    @Override
    public void getDebugInfo(List<String> left, List<String> right, EnumFacing side) {
        left.add("totalSources = " + totalSources);
        boolean added = false;
        for (PlayerPumpInfo info : pumpProgress.values()) {
            if (!added) {
                left.add("Player Progress:");
                added = true;
            }
            left.add("  " + info.profile.getName() + " = " + info.sourcesPumped + " ( "
                + (field_145850_b.func_82737_E() - info.lastPumpTick) / 20 + "s )");
        }
    }

    static class PlayerPumpInfo {
        final GameProfile profile;
        long lastPumpTick = -1;
        int sourcesPumped = 0;

        public PlayerPumpInfo(GameProfile profile) {
            this.profile = profile;
        }

        public PlayerPumpInfo(NBTTagCompound nbt) {
            profile = NBTUtil.func_152459_a(nbt.func_74775_l("profile"));
            lastPumpTick = nbt.func_74763_f("lastPumpTick");
            sourcesPumped = nbt.func_74762_e("sourcesPumped");
        }

        public NBTTagCompound writeToNbt() {
            NBTTagCompound nbt = new NBTTagCompound();
            nbt.func_74782_a("profile", NBTUtil.func_180708_a(new NBTTagCompound(), profile));
            nbt.func_74772_a("lastPumpTick", lastPumpTick);
            nbt.func_74768_a("sourcesPumped", sourcesPumped);
            return nbt;
        }
    }
}
